@@ -44,8 +44,10 @@ enum {
   COOKIE_WIN_MWM_HINTS,
   COOKIE_WIN_HILDON_STACKING,
   COOKIE_WIN_HILDON_TYPE,
+  COOKIE_WIN_PORTRAIT_SUPPORT,
   COOKIE_WIN_PORTRAIT_REQUEST,
   COOKIE_WIN_NO_TRANSITIONS,
+  COOKIE_WIN_LIVE_BACKGROUND,
 
   N_COOKIES
 };
@@ -129,6 +131,7 @@ mb_wm_client_window_init (MBWMObject *this, va_list vap)
 
   win->xwindow = xwin;
   win->wm = wm;
+  win->portrait_supported = win->portrait_requested = -1;
 
   /* TODO: handle properties after discovering them. E.g. fullscreen.
    * See NB#97342 */
@@ -287,6 +290,16 @@ mb_wm_client_window_sync_properties ( MBWMClientWindow *win,
 			      XA_STRING);
     }
 
+  if (props_req & MBWM_WINDOW_PROP_LIVE_BACKGROUND)
+    cookies[COOKIE_WIN_LIVE_BACKGROUND]
+	= mb_wm_property_req (wm,
+			      xwin,
+		      wm->atoms[MBWM_ATOM_HILDON_LIVE_DESKTOP_BACKGROUND],
+			      0,
+			      1L,
+			      False,
+			      XA_INTEGER);
+
   if (props_req & MBWM_WINDOW_PROP_NET_PID)
     {
       cookies[COOKIE_WIN_PID]
@@ -325,8 +338,11 @@ mb_wm_client_window_sync_properties ( MBWMClientWindow *win,
 	  	wm->atoms[MBWM_ATOM_HILDON_STACKING_LAYER]);
     }
 
-  if (props_req & MBWM_WINDOW_PROP_PORTRAIT_REQUEST)
+  if (props_req & MBWM_WINDOW_PROP_PORTRAIT)
     {
+      cookies[COOKIE_WIN_PORTRAIT_SUPPORT]
+	= mb_wm_property_cardinal_req (wm, xwin,
+	  	wm->atoms[MBWM_ATOM_HILDON_PORTRAIT_MODE_SUPPORT]);
       cookies[COOKIE_WIN_PORTRAIT_REQUEST]
 	= mb_wm_property_cardinal_req (wm, xwin,
 	  	wm->atoms[MBWM_ATOM_HILDON_PORTRAIT_MODE_REQUEST]);
@@ -971,6 +987,30 @@ mb_wm_client_window_sync_properties ( MBWMClientWindow *win,
 	}
     }
 
+  if (props_req & MBWM_WINDOW_PROP_LIVE_BACKGROUND)
+    {
+      int *ret;
+      ret = mb_wm_property_get_reply_and_validate (wm,
+				cookies[COOKIE_WIN_LIVE_BACKGROUND],
+						 XA_INTEGER,
+						 32,
+						 1,
+						 NULL,
+						 &x_error_code);
+
+      cookies[COOKIE_WIN_LIVE_BACKGROUND] = 0;
+      if (ret)
+        win->live_background = *ret;
+      else
+        win->live_background = 0;
+
+      if (x_error_code == BadWindow)
+        goto badwindow_error;
+
+      if (ret)
+        XFree (ret);
+    }
+
   if (props_req & MBWM_WINDOW_PROP_NET_PID)
     {
       unsigned char *pid = NULL;
@@ -1162,41 +1202,39 @@ mb_wm_client_window_sync_properties ( MBWMClientWindow *win,
       changes |= MBWM_WINDOW_PROP_HILDON_STACKING;
     }
 
-  if (props_req & MBWM_WINDOW_PROP_PORTRAIT_REQUEST)
+  if (props_req & MBWM_WINDOW_PROP_PORTRAIT)
     {
-      unsigned char *value = NULL;
-
-      mb_wm_property_reply (wm,
-			    cookies[COOKIE_WIN_PORTRAIT_REQUEST],
-			    &actual_type_return,
-			    &actual_format_return,
-			    &nitems_return,
-			    &bytes_after_return,
-			    &value,
-			    &x_error_code);
-
-      cookies[COOKIE_WIN_PORTRAIT_REQUEST] = 0;
-
-      if (x_error_code
-	  || actual_type_return != XA_CARDINAL
-	  || actual_format_return != 32
-	  || value == NULL
-	  )
-	{
-          if (x_error_code == BadWindow)
-	    {
-              if (value)
-	        XFree(value);
-              goto badwindow_error;
-	    }
-	}
-      else
-        win->portrait_on_map = *(unsigned *)value;
+      unsigned char *value;
 
+      value = NULL;
+      mb_wm_property_reply (wm, cookies[COOKIE_WIN_PORTRAIT_SUPPORT],
+			    &actual_type_return, &actual_format_return,
+			    &nitems_return, &bytes_after_return,
+			    &value, &x_error_code);
+      cookies[COOKIE_WIN_PORTRAIT_SUPPORT] = 0;
+      win->portrait_supported = !x_error_code && value
+          && actual_type_return == XA_CARDINAL && actual_format_return == 32
+        ? *(unsigned *)value : -1;
       if (value)
-	XFree(value);
+        XFree(value);
+      if (x_error_code == BadWindow)
+          goto badwindow_error;
 
-      changes |= MBWM_WINDOW_PROP_PORTRAIT_REQUEST;
+      value = NULL;
+      mb_wm_property_reply (wm, cookies[COOKIE_WIN_PORTRAIT_REQUEST],
+			    &actual_type_return, &actual_format_return,
+			    &nitems_return, &bytes_after_return,
+			    &value, &x_error_code);
+      cookies[COOKIE_WIN_PORTRAIT_REQUEST] = 0;
+      win->portrait_requested = !x_error_code && value
+        && actual_type_return == XA_CARDINAL && actual_format_return == 32
+        ? *(unsigned *)value : -1;
+      if (value)
+        XFree(value);
+      if (x_error_code == BadWindow)
+          goto badwindow_error;
+
+      changes |= MBWM_WINDOW_PROP_PORTRAIT;
     }
 
   if (changes)
@@ -1266,6 +1304,20 @@ badwindow_error:
         XFree (wmhints);
     }
 
+  if (cookies[COOKIE_WIN_LIVE_BACKGROUND])
+    {
+      int *ret;
+      ret = mb_wm_property_get_reply_and_validate (wm,
+		                cookies[COOKIE_WIN_LIVE_BACKGROUND],
+			        XA_INTEGER,
+			        32,
+			        1,
+			        NULL,
+			        &x_error_code);
+      if (ret)
+        XFree (ret);
+    }
+
   if (cookies[COOKIE_WIN_MWM_HINTS])
     {
       MotifWmHints *mwmhints;
@@ -1315,6 +1367,7 @@ badwindow_error:
                         COOKIE_WIN_USER_TIME,
                         COOKIE_WIN_CM_TRANSLUCENCY,
                         COOKIE_WIN_HILDON_STACKING,
+                        COOKIE_WIN_PORTRAIT_SUPPORT,
                         COOKIE_WIN_PORTRAIT_REQUEST,
                         COOKIE_WIN_NET_STATE,
                         COOKIE_WIN_TYPE,
