@@ -41,6 +41,7 @@ struct MBWindowManagerClientPriv
   Bool          unmap_confirmed;
   Bool          iconizing;
   Bool          hiding_from_desktop;
+  Bool          geometry_requested;
   MBWMSyncType  sync_state;
 };
 
@@ -169,12 +170,7 @@ mb_wm_client_init (MBWMObject *obj, va_list vap)
 
     if (format && format->type == PictTypeDirect &&
 	format->direct.alphaMask)
-      {
-        MBWMClientType ctype = MB_WM_CLIENT_CLIENT_TYPE (client);
-        if (ctype != MBWMClientTypeApp &&
-            ctype != MBWMClientTypeDialog)
-          client->is_argb32 = True;
-      }
+      client->is_argb32 = True;
   }
 #endif
 
@@ -315,6 +311,20 @@ mb_wm_client_on_property_change (MBWMClientWindow        *window,
     }
 #endif
 
+  if (property & MBWM_WINDOW_PROP_PORTRAIT)
+    {
+      client->portrait_supported            = window->portrait_supported > 0;
+      client->portrait_supported_inherited  = window->portrait_supported < 0;
+      client->portrait_requested            = window->portrait_requested > 0
+                                              ? window->portrait_requested : 0;
+      client->portrait_requested_inherited  = window->portrait_requested < 0;
+      g_debug ("portrait properties of %p: supported=%d requested=%d", client,
+               client->portrait_supported_inherited
+                 ? -1 : client->portrait_supported,
+               client->portrait_requested_inherited
+                 ? -1 : client->portrait_requested);
+    }
+
   return False;
 }
 
@@ -363,6 +373,14 @@ mb_wm_client_stack (MBWindowManagerClient *client,
 {
   MBWindowManagerClientClass *klass;
 
+  if (client->window->live_background)
+    {
+      /* stack live background to the bottom */
+      mb_wm_stack_remove (client);
+      mb_wm_stack_prepend_bottom (client);
+      return;
+    }
+
   klass = MB_WM_CLIENT_CLASS (MB_WM_OBJECT_GET_CLASS (client));
 
   if (klass->stack)
@@ -375,12 +393,6 @@ mb_wm_client_stack (MBWindowManagerClient *client,
     }
 }
 
-Bool
-mb_wm_client_needs_stack_sync (MBWindowManagerClient *client)
-{
-  return (client->priv->sync_state & MBWMSyncStacking);
-}
-
 void
 mb_wm_client_show (MBWindowManagerClient *client)
 {
@@ -497,6 +509,12 @@ mb_wm_client_is_unmap_confirmed (MBWindowManagerClient *client)
   return client->priv->unmap_confirmed;
 }
 
+Bool
+mb_wm_client_is_geometry_requested (MBWindowManagerClient *client)
+{
+  return client->priv->geometry_requested;
+}
+
 void
 mb_wm_client_set_unmap_confirmed (MBWindowManagerClient *client,
                                   Bool confirmed)
@@ -524,13 +542,16 @@ mb_wm_client_request_geometry (MBWindowManagerClient *client,
 			       MBWMClientReqGeomType  flags)
 {
   MBWindowManagerClientClass *klass;
+  Bool ret = False;
 
   klass = MB_WM_CLIENT_CLASS (MB_WM_OBJECT_GET_CLASS (client));
 
   if (klass->geometry)
-    return klass->geometry(client, new_geometry, flags);
+    ret = klass->geometry(client, new_geometry, flags);
 
-  return False;
+  client->priv->geometry_requested = TRUE;
+
+  return ret;
 }
 
 void
@@ -673,14 +694,14 @@ mb_wm_client_get_last_focused_transient (MBWindowManagerClient *client)
 
   mb_wm_stack_enumerate_reverse (wm, c)
     if (c->transient_for == client &&
-        c->window->net_type != 
+        c->window->net_type !=
 	  wm->atoms[MBWM_ATOM_HILDON_WM_WINDOW_TYPE_ANIMATION_ACTOR])
       {
         result = c;
         break;
       }
 
-  if (result) 
+  if (result)
     {
       MBWindowManagerClient  *rec_transient;
 
@@ -1138,6 +1159,10 @@ mb_wm_client_get_stacking_layer (MBWindowManagerClient *client)
 {
   MBWindowManagerClientClass *klass;
 
+  if (client->window->live_background)
+    /* stack live background to the bottom */
+    return MBWMStackLayerUnknown;
+
   klass = MB_WM_CLIENT_CLASS (MB_WM_OBJECT_GET_CLASS (client));
 
   if (klass->stacking_layer)
@@ -1259,7 +1284,7 @@ mb_wm_client_covers_screen (MBWindowManagerClient * client)
 Bool
 mb_wm_client_wants_portrait (MBWindowManagerClient * client)
 {
-  if (!client->window->portrait_on_map)
+  if (client->window->portrait_requested <= 0)
     /* Out of scope. */
     return False;
 
@@ -1269,10 +1294,41 @@ mb_wm_client_wants_portrait (MBWindowManagerClient * client)
     return False;
 
   if ((MB_WM_CLIENT_CLIENT_TYPE (client) & MBWMClientTypeDialog)
-      && client->window->portrait_on_map < 2)
-    /* But only dialogs which demant rotation. */
+      && client->window->portrait_requested < 2)
+    /* But only dialogs which demand rotation. */
     return False;
 
   /* If we cannot say for sure hd will decide. */
   return True;
 }
+
+/* Update the inherited portrait flags of @cs if they were calculated
+ * earlier than @now.  If @now is G_MAXUINT the flags are uncoditionally
+ * updated but are not cached. */
+void
+mb_wm_client_update_portrait_flags (MBWindowManagerClient *cs, guint now)
+{
+  MBWindowManagerClient *ct;
+
+  if ((cs->portrait_supported_inherited
+       || cs->portrait_requested_inherited)
+      && cs->portrait_timestamp != now)
+    { /* @cs has outdated flags */
+      if (  !cs->portrait_requested_inherited
+          && cs->portrait_requested
+          && cs->portrait_supported_inherited)
+        /* Add some crap to the pile: if you request but don't say
+         * you support you do. */
+        cs->portrait_supported = TRUE;
+      else if (cs->transient_for)
+        { /* Get the parent's and copy them. */
+          mb_wm_client_update_portrait_flags (ct = cs->transient_for, now);
+          if (cs->portrait_supported_inherited)
+            cs->portrait_supported = ct->portrait_supported;
+          if (cs->portrait_requested_inherited)
+            cs->portrait_requested = ct->portrait_requested;
+        }
+      if (now != G_MAXUINT)
+        cs->portrait_timestamp = now;
+    }
+}
