@@ -206,7 +206,7 @@ mb_wm_decor_release_handler (XButtonEvent    *xev,
 {
   MBWMDecor       *decor  = userdata;
   MBWindowManager *wm;
- 
+
   if (decor == NULL || decor->parent_client == NULL
       || decor->parent_client->wmref == NULL)
     return False;
@@ -237,6 +237,7 @@ mb_wm_decor_press_handler (XButtonEvent    *xev,
       MBGeometry geom;
       int        orig_x, orig_y;
       int        orig_p_x, orig_p_y;
+      int        status;
 
       mb_wm_client_get_coverage (decor->parent_client, &geom);
 
@@ -256,12 +257,15 @@ mb_wm_decor_press_handler (XButtonEvent    *xev,
        * ButtonPress we install a ButtonRelease callback into the main loop
        * and use that to release the grab.
        */
-      if (XGrabPointer(wm->xdpy, xev->subwindow, False,
+      mb_wm_util_async_trap_x_errors(wm->xdpy);
+      status = XGrabPointer(wm->xdpy, xev->subwindow, False,
 		       ButtonPressMask|ButtonReleaseMask|
 		       PointerMotionMask|EnterWindowMask|LeaveWindowMask,
 		       GrabModeAsync,
 		       GrabModeAsync,
-		       None, None, CurrentTime) == GrabSuccess)
+		       None, None, CurrentTime);
+      mb_wm_util_async_untrap_x_errors();
+      if (status == GrabSuccess)
 	{
 
 	  decor->release_cb_id = mb_wm_main_context_x_event_handler_add (
@@ -354,6 +358,7 @@ mb_wm_decor_sync_window (MBWMDecor *decor)
       attr.background_pixmap = None;
       attr.event_mask = ButtonPressMask|ButtonReleaseMask|ButtonMotionMask;
 
+      XSync(wm->xdpy, False);
       mb_wm_util_trap_x_errors();
 
       decor->xwin
@@ -410,7 +415,7 @@ mb_wm_decor_sync_window (MBWMDecor *decor)
   else
     {
       /* Resize */
-      mb_wm_util_trap_x_errors();
+      mb_wm_util_async_trap_x_errors_warn(wm->xdpy, "XMoveResizeWindow");
 
       XMoveResizeWindow(wm->xdpy,
 			decor->xwin,
@@ -419,14 +424,12 @@ mb_wm_decor_sync_window (MBWMDecor *decor)
 			decor->geom.width,
 			decor->geom.height);
 
-      /* Next up sort buttons */
+      mb_wm_util_async_untrap_x_errors();
 
+      /* Next up sort buttons */
       mb_wm_util_list_foreach(decor->buttons,
 			      (MBWMListForEachCB)mb_wm_decor_button_sync_window,
 			      NULL);
-
-      if (mb_wm_util_untrap_x_errors())
-	return False;
     }
 
   return True;
@@ -446,7 +449,7 @@ mb_wm_decor_reparent (MBWMDecor *decor)
 
   /* FIXME: Check if we already have a parent here */
 
-  mb_wm_util_trap_x_errors();
+  mb_wm_util_async_trap_x_errors_warn(wm->xdpy, "XReparentWindow");
 
   XReparentWindow (wm->xdpy,
 		   decor->xwin,
@@ -454,8 +457,7 @@ mb_wm_decor_reparent (MBWMDecor *decor)
 		   decor->geom.x,
 		   decor->geom.y);
 
-  if (mb_wm_util_untrap_x_errors())
-    return False;
+  mb_wm_util_async_untrap_x_errors();
 
   return True;
 }
@@ -691,16 +693,16 @@ mb_wm_decor_destroy (MBWMObject* obj)
   if (decor->press_cb_id)
     mb_wm_main_context_x_event_handler_remove (ctx, ButtonPress,
 					       decor->press_cb_id);
-  
+
   if (decor->release_cb_id)
     mb_wm_main_context_x_event_handler_remove (ctx, ButtonRelease,
 					     decor->release_cb_id);
 
   if (decor->xwin != None)
     {
-      mb_wm_util_trap_x_errors();
+      mb_wm_util_async_trap_x_errors(decor->parent_client->wmref->xdpy);
       XDestroyWindow (decor->parent_client->wmref->xdpy, decor->xwin);
-      mb_wm_util_untrap_x_errors();
+      mb_wm_util_async_untrap_x_errors();
     }
 
   memset (decor, 0, sizeof (*decor));
@@ -796,7 +798,7 @@ mb_wm_decor_button_press_handler (XButtonEvent    *xev,
     return False;
 
   wm = decor->parent_client->wmref;
-  
+
   mb_wm_object_ref (MB_WM_OBJECT(button));
 
   if (xev->window == decor->xwin)
@@ -873,6 +875,7 @@ mb_wm_decor_button_press_handler (XButtonEvent    *xev,
       else
 	{
 	  XEvent ev;
+	  int status;
 
 	  /*
 	   * First, call the custom function if any.
@@ -880,12 +883,15 @@ mb_wm_decor_button_press_handler (XButtonEvent    *xev,
 	  if (button->press)
 	    button->press(wm, button, button->userdata);
 
-	  if (XGrabPointer(wm->xdpy, xev->subwindow, False,
+	  mb_wm_util_async_trap_x_errors(wm->xdpy);
+	  status = XGrabPointer(wm->xdpy, xev->subwindow, False,
 			   ButtonPressMask|ButtonReleaseMask|
 			   PointerMotionMask|EnterWindowMask|LeaveWindowMask,
 			   GrabModeAsync,
 			   GrabModeAsync,
-			   None, None, CurrentTime) == GrabSuccess)
+			   None, None, CurrentTime);
+	  mb_wm_util_async_untrap_x_errors();
+	  if (status == GrabSuccess)
 	    {
               /* set up release handler to catch ButtonRelease while we
                * are spinning the main loop */
@@ -1207,7 +1213,7 @@ mb_wm_decor_button_unrealize (MBWMDecorButton *button)
   ctx = button &&
 	  button->decor &&
 	  button->decor->parent_client &&
-	  button->decor->parent_client->wmref ? 
+	  button->decor->parent_client->wmref ?
 	  button->decor->parent_client->wmref->main_ctx : NULL;
 
   if (!ctx)
